//
// vibra.cpp
//
// Copyright 2005 Helsinki Institute for Information Technology (HIIT)
// and the authors.  All rights reserved.
//
// Authors: Tero Hasu <tero.hasu@hut.fi>
// Paul Wisner - extracted from PyExt project to create minimal example.
// Paul Wisner - simplified by removing MVibraControlObserver impl.

// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software,
// and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
// BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
// ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include <vibractrl.h> // CVibraControl
#include "Python.h"
#include "symbian_python_ext_util.h"

class CPyExtVibra : public CBase, public MVibraControlObserver
{
public:
  static CPyExtVibra* NewLC();
  void ConstructL();
  ~CPyExtVibra();
private:
  CPyExtVibra();
public:
  void VibrateL(TUint16 aDuration, TInt aIntensity); // synchronous
private:
  CVibraControl* iVibraControl;

// This stuff should not be necessary because the doc sais we can ignore results
private: // MVibraControlObserver
  void VibraModeStatus(CVibraControl::TVibraModeState /*aStatus*/) {}
  void VibraRequestStatus(CVibraControl::TVibraRequestStatus aStatus);
private:
  CActiveSchedulerWait* iLoop;
  TRequestStatus iStatus;
};

CPyExtVibra* CPyExtVibra::NewLC()
{
  CPyExtVibra* taker = new (ELeave) CPyExtVibra;
  CleanupStack::PushL(taker);
  taker->ConstructL();
  return taker;
}

CPyExtVibra::CPyExtVibra()

{
  // nothing
}

void CPyExtVibra::ConstructL()
{  
	// DOTO:  instead of nested scheduler try using CActiveScheduler::Add ??? 
  iLoop = new (ELeave) CActiveSchedulerWait();
  iVibraControl = VibraFactory::NewL(this);
}

CPyExtVibra::~CPyExtVibra()
{
  delete iVibraControl;
  delete iLoop;
}

void CPyExtVibra::VibrateL(TUint16 aDuration, TInt aIntensity)
{
  iStatus = KRequestPending;
  iVibraControl->StartVibraL(aDuration, aIntensity);
  if (iStatus == KRequestPending) {
    if (iLoop) iLoop->Start();
  }
}

static void DoVibrateL(TUint16 aDuration, TInt aIntensity)
{
  CPyExtVibra* vibra = CPyExtVibra::NewLC();
  vibra->VibrateL(aDuration, aIntensity);
  CleanupStack::PopAndDestroy();
}


/**(uw new_module_method "vibrate", 
   :args => [[:int, "duration"], [:int, "intensity"]],
   :doc =>
   "<p>Runs the vibra motor for the specified duration of time
    at the specified intensity. The duration is specified
    in milliseconds, and must be greater than zero.
    The intensity is a value between -100 and 100 (inclusive),
    and it specifies percentage of full rotation speed.
    Positive and negative values cause rotation in opposite directions;
    a zero value indicates no rotation, and thus it makes little
    sense to call this function with 0 intensity argument.</p>
    <p>Vibration will not work unless enabled in the profile.
    Only supported on Series 60 2nd Ed FP2 and higher.</p>" */
static PyObject* miso_Vibrate(PyObject* /*self*/, PyObject* args)
{
  TInt durationInt, intensity;
  if (!PyArg_ParseTuple(args, "ii", &durationInt, &intensity))
  {
    return NULL;
  }

  TUint16 duration = static_cast<TUint16>(durationInt);
  if ((duration <= 0) ||
      ((intensity < -100 || intensity > 100)))
      return SPyErr_SetFromSymbianOSErr(KErrArgument);

  TInt error;
  Py_BEGIN_ALLOW_THREADS;
  TRAP(error, DoVibrateL(duration, intensity));
  Py_END_ALLOW_THREADS;
  if (error)
      return SPyErr_SetFromSymbianOSErr(error);
  return Py_BuildValue("i", 1);
}

static const PyMethodDef vibra_methods[] = {
/*******************************************************/
    {"vibrate", (PyCFunction)miso_Vibrate, METH_VARARGS, "Runs the vibra motor for the specified duration of time at the specified intensity. The duration is specified in milliseconds, and must be greater than zero. The intensity is a value between -100 and 100 (inclusive), and it specifies percentage of full rotation speed. Positive and negative values cause rotation in opposite directions; a zero value indicates no rotation, and thus it makes little sense to call this function with 0 intensity argument. Vibration will not work unless enabled in the profile. Only supported on Series 60 2nd Ed FP2 and higher."},
    {0, 0}
};


DL_EXPORT(void) MODULE_INIT_FUNC()
{
  /* PyObject* module = */
    Py_InitModule("vibra", vibra_methods);
}


/*******************************************************
 This function is mandatory in Symbian DLL's. */

GLDEF_C TInt E32Dll(TDllReason)
{
  return KErrNone;
}


//************************************************
//************************************************
//************************************************
//************************************************
//************************************************
void CPyExtVibra::VibraRequestStatus(CVibraControl::TVibraRequestStatus aStatus)
{
  if (aStatus == CVibraControl::EVibraRequestOK)
    return; // wait until vibra stops

  TInt error = KErrUnknown;
  if (aStatus == CVibraControl::EVibraRequestFail) {
    error = KErrGeneral;
  } else if (aStatus == CVibraControl::EVibraRequestNotAllowed) {
    error = KErrAccessDenied;
  } else if (aStatus == CVibraControl::EVibraRequestStopped) {
    error = KErrNone;
  } else if (aStatus == CVibraControl::EVibraRequestUnableToStop) {
    // signal completion anyway
  } else if (aStatus == CVibraControl::EVibraRequestUnknown) {
    // signal completion anyway
  }

  iStatus = error;

  if (iLoop && iLoop->IsStarted()) {
    iLoop->AsyncStop();
  }
}
